unit UnitFrameHotKey;
{


}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, Buttons, UnitFrmHotKey;

{
type TKeystrokeData = record
    alt : boolean;
    ctrl : boolean;
    shft : boolean;
    win : boolean;
    key : word;
    name : string;
end;
}
type
  TFrameKeystrokeEdit = class(TFrame)
    bSet: TButton;
    bCancel: TButton;
    txtHotkey: TEdit;
    Label1: TLabel;
    btnClear: TButton;
    SpeedButton1: TSpeedButton;
    pmKeys: TPopupMenu;
    CloseAltF41: TMenuItem;
    btnTab: TButton;
    NextControlAltTAB1: TMenuItem;
    askManager1: TMenuItem;
    MinimizeAllWinkeyM1: TMenuItem;
    N1: TMenuItem;
    Clear1: TMenuItem;
    SelectallCTRLA1: TMenuItem;
    btnWinkey: TButton;
    procedure txtHotkeyKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure txtHotkeyKeyPress(Sender: TObject; var Key: Char);
    procedure bSetClick(Sender: TObject);
    procedure bCancelClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnClearClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure btnTabClick(Sender: TObject);
    procedure CloseAltF41Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure txtHotkeyEnter(Sender: TObject);
    procedure btnWinkeyClick(Sender: TObject);
  private
    { Private declarations }
    Hotkey_Ctrl : boolean;
    Hotkey_Alt : boolean;
    Hotkey_Winkey : boolean;
    Hotkey_Shift : boolean;
    Hotkey_Key : word;
    ClearPressed : boolean;
    SetPressed : boolean;
  public
    { Public declarations }
        function FromString(s: string): TKeystrokeData;
        function ToString(hkd: TKeystrokeData): string;
        function GetHotkey(var kAlt, kShift, kCtrl, kWinkey : boolean;
            var VKCode : word;
            var ReadableName : string; ShowClearButton : boolean = false ) : boolean; overload;
        function GetHotKey(var keydata : TKeystrokeData; ShowClearButton : boolean = false) : boolean; overload;

        function GetHumanReadable( kAlt, kShift, kCtrl, kWinkey : boolean;
                    VKCode : word; var Valid : boolean) : string; overload;

        function GetHumanReadable(keydata : TKeystrokeData; var valid : boolean) : string; overload;

        procedure SetKeystroke( s : string);
        procedure Init;
  end;



var
  FrameKeystrokeEdit: TFrameKeystrokeEdit;

implementation

const INVALID_KEYS : string = 'INVALID';
const EMPTY_HOTKEY_NAME = 'none';
{$R *.dfm}



function TFrameKeystrokeEdit.ToString(hkd: TKeystrokeData): string;
begin
    result :=
        IntToStr(Integer(hkd.alt)) +
        IntToStr(Integer(hkd.ctrl)) +
        IntToStr(Integer(hkd.shft)) +
        IntToStr(Integer(hkd.win)) +
        IntToHex(hkd.key,2) +
        hkd.name;

end;



function TFrameKeystrokeEdit.FromString(s: string): TKeystrokeData;
begin
    result.alt := false;
    result.ctrl := false;
    result.shft := false;
    result.win := false;
    result.key := 0;
    result.name := EMPTY_HOTKEY_NAME;
    if length(s) < 6 then begin
        EXIT;
    end;

    result.alt := Boolean(StrToInt(s[1]));
    result.ctrl  := Boolean(StrToInt(s[2]));
    result.shft  := Boolean(StrToInt(s[3]));
    result.win  := Boolean(StrToInt(s[4]));
    result.key := StrToInt('$' +  copy(s,5,2));
    delete(s,1,6);
    result.name := s;
end;

//
// Returns: MOD+KEYNAME when valid
// Returns: "INVALID" on illegal combination
//
function TFrameKeystrokeEdit.GetHumanReadable( kAlt, kShift, kCtrl, kWinkey : boolean;
    VKCode : word; var Valid : boolean) : string;
var s : string;
    HasMod, HasKey : boolean;
    const punc1 : string = ';=,-./`';
    const punc2 : string = '[\]''';

begin
    HasMod := false;
    if kWinkey = true then begin
        s := s + 'WINKEY+';
        HasMod := true;
    end;

    if kCtrl = true then begin
        s := s + 'CTRL+';
        HasMod := true;
    end;

    if kShift = true then begin
        s := s + 'SHIFT+';
        HasMod := true;
    end;


    if kAlt = true then begin
        s := s + 'ALT+';
        HasMod := true;
    end;

    //
    // Determine the non-modifier key pressed
    //

    HasKey := true;
    case VKCode of

    VK_BACK:    begin s := s + 'BACKSPACE';  end;
    VK_ESCAPE:  begin s := s + 'ESCAPE';  end;
    VK_TAB:     begin s := s + 'TAB';  end;
    VK_RETURN:  begin s := s + 'ENTER'; end;
    VK_PRINT:   begin s := s + 'PRINT';  end;
    VK_PAUSE:   begin s := s + 'PAUSE';  end;
    VK_SPACE:   begin s := s + 'SPACE';  end;
    VK_HOME:    begin s := s + 'HOME';  end;
    VK_END:     begin s := s + 'END'; end;
    VK_DECIMAL: begin s := s + 'DECIMAL';  end;
    VK_INSERT:  begin s := s + 'INSERT';  end;
    VK_DELETE:  begin s := s + 'DELETE';  end;
    VK_PRIOR:   begin s := s + 'PAGEUP';  end;
    VK_NEXT:    begin s := s + 'PAGEDOWN';  end;
    VK_DOWN:    begin s := s + 'DOWN';  end;
    VK_UP:      begin s := s + 'UP';  end;
    VK_LEFT:    begin s := s + 'LEFT';  end;
    VK_RIGHT:   begin s := s + 'RIGHT';  end;

    VK_SUBTRACT:begin s := s + 'SUBTRACT';  end;
    VK_ADD:     begin s := s + 'ADD';  end;
    VK_MULTIPLY:begin s := s + 'MULTIPLY';  end;
    VK_DIVIDE:  begin s := s + 'DIVIDE';  end;
    VK_NUMLOCK: begin s := s + 'NUMLOCK';  end;

    // 0 .. 9
    48..57:     begin s := s + chr(VKCode);
                end;
    // A .. Z
    65..90:     begin s := s + chr(VKCode);
                end;
    // numpad #
    96..105:    begin s := s + 'NUMPAD ' + IntToStr(VKCode - 95);
                end;
    // F1 .. F12
    112..123:   begin s := s + 'F' + IntToStr(VKCode-111);
                end;
    // punctuation VKCodes
    186..192:   begin
                    s := s + punc1[VKCode-185];
                end;
    // []\` VKCodes
    219..222:   begin
                    s := s + punc2[VKCode-218];
                end;
    else
        HasKey := false;
    end;

    result := s;
    valid := (HasMod and HasKey);
end;

//
// Return true if new hotkey was selected
//
function TFrameKeystrokeEdit.GetHotkey(var kAlt, kShift, kCtrl, kWinkey : boolean;
    var VKCode : word;
    var ReadableName : string; ShowClearButton : boolean = false) : boolean;
var b : boolean;
begin
    //SetPressed := false;
    //ClearPressed := false;
    //btnClear.Visible := ShowClearButton;
    //btnClear.click;
    //self.ShowModal;

    //if SetPressed then begin
        kAlt := Hotkey_Alt;
        kShift := Hotkey_Shift;
        kCtrl := Hotkey_Ctrl;
        kWinkey := Hotkey_Winkey;
        VKCode := Hotkey_key;
        ReadableName := self.GetHumanReadable(kAlt, kShift, kCtrl, kWinkey, VKCode, b);
    //end;

    {if ClearPressed then begin
        kAlt := false;
        kShift := false;
        kCtrl := false;
        kWinkey := false;
        VKCode := 0;
        ReadableName := '';
    end;
    result := SetPressed or ClearPressed;}
end;


procedure TFrameKeystrokeEdit.txtHotkeyEnter(Sender: TObject);
begin
    txtHotkey.SelLength := 0;
end;

procedure TFrameKeystrokeEdit.txtHotkeyKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var State : TKeyboardState;
    s : string;
    valid : boolean;
begin
    bSet.Enabled := false;

    //Hotkey_alt := false;
    //Hotkey_Ctrl := false;
    //Hotkey_Shift := false;
    //Hotkey_Winkey := false;
    
    //
    // check modifier keys
    //
    Windows.GetKeyboardState(State);

    if ((State[vk_rwin] and 128) <> 0) or ((State[vk_lwin] and 128) <> 0)then begin
        Hotkey_Winkey := true;
        s := s + 'WINKEY+';
    end;

    if ((State[vk_control] and 128) <> 0) then begin
        Hotkey_Ctrl := true;
        s := s + 'CTRL+';
    end;


    if ((State[vk_shift] and 128) <> 0) then begin
        Hotkey_Shift := true;
        s := s + 'SHIFT+';
    end;


    if ((State[vk_menu] and 128) <> 0) then begin
        Hotkey_Alt := true;
        s := s + 'ALT+';
    end;

    // workaround for manual insetrs of keystrokes
    case key of
    VK_CONTROL : begin s := s + 'CTRL+'; hotkey_Ctrl := true end;
    VK_SHIFT : begin s := s + 'SHIFT+'; hotkey_Shift := true; end;
    VK_LWIN : begin s := s + 'WIN+'; Hotkey_Winkey := true; end;
    VK_MENU  : begin s := s + 'ALT+'; Hotkey_Alt := true; end;
    end;

    s := Self.GetHumanReadable(hotkey_alt, hotkey_shift, hotkey_ctrl, hotkey_winkey, key, valid);
    //
    // Final results
    // Only allow button to be pressed when we have a modifier and
    // a key selected
    //
    valid := true;
    txtHotkey.text := s;
    bSet.Enabled := Valid;
    btnClear.Enabled := valid;
    Hotkey_Key := key;
    key := 0;
end;

procedure TFrameKeystrokeEdit.txtHotkeyKeyPress(Sender: TObject; var Key: Char);
begin
    key := #0;
end;

procedure TFrameKeystrokeEdit.bSetClick(Sender: TObject);
begin
    SetPressed := true;
    //self.Close;
end;

procedure TFrameKeystrokeEdit.bCancelClick(Sender: TObject);
begin
    //self.Close;
end;

procedure TFrameKeystrokeEdit.FormActivate(Sender: TObject);
begin
    //self.ActiveControl := txtHotkey;
end;

procedure TFrameKeystrokeEdit.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    {if (KEY = VK_ESCAPE) then begin
        self.bCancel.Click;
    end;}
end;

procedure TFrameKeystrokeEdit.btnClearClick(Sender: TObject);
begin
    Hotkey_alt := false;
    Hotkey_Ctrl := false;
    Hotkey_Shift := false;
    Hotkey_Winkey := false;
    Hotkey_Key := 0;
    txtHotkey.text := '';

    if self.Visible and txtHotkey.CanFocus then
        txtHotkey.SetFocus;
end;

procedure TFrameKeystrokeEdit.btnTabClick(Sender: TObject);
var w : word;
begin
    w := VK_TAB;
    self.txtHotkeyKeyDown(sender, w, []);
    txtHotkey.SetFocus;
end;

procedure TFrameKeystrokeEdit.btnWinkeyClick(Sender: TObject);
var w : word;
begin
    w := VK_LWIN;
    self.txtHotkeyKeyDown(sender, w, []);
    txtHotkey.SetFocus;
end;

procedure TFrameKeystrokeEdit.Clear1Click(Sender: TObject);
begin
    btnClear.Click;
end;

procedure TFrameKeystrokeEdit.CloseAltF41Click(Sender: TObject);
var kd : TKeystrokeData;
    w : word;
    s : TShiftState;
begin
    kd := self.FromString( TMEnuItem(sender).hint );
    self.btnClear.click;


    if kd.win then begin
        w := VK_LWIN;;
        self.txtHotkeyKeyDown(sender, w, []);
    end;
    if kd.alt then  begin
        w := VK_MENU;
        self.txtHotkeyKeyDown(sender, w, []);
    end;
    if kd.ctrl then  begin
        w := VK_CONTROL;
        self.txtHotkeyKeyDown(sender, w, []);
    end;
    if kd.shft then  begin
        w := VK_SHIFT;
        self.txtHotkeyKeyDown(sender, w, []);
    end;
    self.txtHotkeyKeyDown(sender, kd.key , s);
    
end;

function TFrameKeystrokeEdit.GetHotkey(var keydata: TKeystrokeData;
  ShowClearButton: boolean): boolean;
begin
    with keydata do begin
        result := self.GetHotkey(alt,shft,ctrl,win,key,name,ShowClearButton);
    end;
end;

function TFrameKeystrokeEdit.GetHumanReadable(keydata: TKeystrokeData;
  var valid: boolean): string;
begin
    with keydata do begin
        self.GetHumanReadable(alt,shft,ctrl,win,key,valid);
    end;
end;

procedure TFrameKeystrokeEdit.Init;
begin
     btnClear.click;
end;

procedure TFrameKeystrokeEdit.SetKeystroke(s: string);
var kd : TKeystrokeData;
    b : boolean;
begin
    kd := self.FromString(s);
    Hotkey_Ctrl := kd.ctrl;
    Hotkey_Alt := kd.alt;
    Hotkey_Winkey := kd.win;
    Hotkey_Shift := kd.shft;
    Hotkey_Key := kd.key;
    txtHotkey.text := kd.name;
end;

procedure TFrameKeystrokeEdit.SpeedButton1Click(Sender: TObject);
begin
    pmKeys.Popup(mouse.CursorPos.X +5, mouse.CursorPos.Y + 7);
end;

end.
