unit UnitFrmGestureWizard;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, UnitFrameHotKey, ComCtrls, Buttons, ExtCtrls, StdCtrls,
  UnitFrameGestureEdit, UnitFrameKeystrokeList, UnitFrameProgramEdit, UnitFrmGestures, UnitFrmHotKey ;

type
  TFrmGestureWizard = class(TForm)
    Panel1: TPanel;
    btnBack: TSpeedButton;
    btnNext: TSpeedButton;
    btnFinish: TSpeedButton;
    pcPages: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    FrameKeystrokeEdit1: TFrameKeystrokeEdit;
    FrameGestureEdit1: TFrameGestureEdit;
    rbSingle: TRadioButton;
    rbMultiple: TRadioButton;
    rbRun: TRadioButton;
    Label1: TLabel;
    TabSheet4: TTabSheet;
    TabSheet5: TTabSheet;
    TabSheet6: TTabSheet;
    Label2: TLabel;
    Shape1: TShape;
    Label3: TLabel;
    Label5: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    FrameKeystrokeList1: TFrameKeystrokeList;
    mLines: TMemo;
    Label11: TLabel;
    Label12: TLabel;
    FrameProgramEdit1: TFrameProgramEdit;
    procedure FormShow(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure btnFinishClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
    afterfirst : boolean;
    EditMode : boolean;
    fGesture : string;
    fGestureName : string;
    fActionData : string;
    fActionType : TActionType;
    procedure SetButtons;
    procedure InitPages;
    procedure InitPage;
    procedure SetGesture(const Value: string);
    procedure SetActionType(const Value: TActionType);
    procedure SetActionData(const Value: string);
  public
    { Public declarations }
    property Gesture : string read fGesture write SetGesture;
    property GestureName : string read fGestureName write fGestureName;
    property ActionType : TActionType read factiontype write SetActionType;
    property ActionData : string read fActionData write SetActionData;
    function NewGesture : boolean;
    function EditGesture(DataPage : boolean = false) : boolean;
  end;

var
  FrmGestureWizard: TFrmGestureWizard;

implementation



{$R *.dfm}

procedure TFrmGestureWizard.btnBackClick(Sender: TObject);
var i : integer;
begin
   i := pcPages.ActivePageIndex;
    repeat
        dec(i);
    until pcPages.pages[i].Enabled;
    pcPages.ActivePageIndex := i;

    self.InitPage;
    self.SetButtons;
end;

procedure TFrmGestureWizard.btnFinishClick(Sender: TObject);
begin
    self.ModalResult := mrOK;
end;

procedure TFrmGestureWizard.btnNextClick(Sender: TObject);
var i : integer;
    gestname : string;
begin
    case pcPages.ActivePageIndex of
    0: begin
        if not EditMode then begin
            FrameGestureEdit1.GetGesture(gestname, self.fGesture);
            if FrmGestures.lvGestures.FindCaption(0,gestname,false,true,false) <> nil then begin
                ShowMessage('Cannot use '+gestname+'. Gesture already exists.');
                EXIT;
            end;
        end;

    end;
    1: begin
        pcPages.Pages[2].Enabled  := false;
        pcPages.Pages[3].Enabled := false;
        pcPages.Pages[4].Enabled := false;

        if rbSingle.checked then begin
            pcPages.Pages[2].Enabled := true;
        end else if rbMultiple.Checked then begin
            pcPages.Pages[3].Enabled := true;
        end else if rbRun.checked then begin
            pcPages.Pages[4].Enabled := true;
        end;
    end;
    end;

    i := pcPages.ActivePageIndex;
    repeat
        inc(i);
    until pcPages.pages[i].Enabled;
    pcPages.ActivePageIndex := i;



    self.InitPage;
    self.SetButtons;
end;

function TFrmGestureWizard.EditGesture(DataPage : boolean = false): boolean;
begin
    EditMode := true;

    if DataPage then begin
        pcPages.ActivePageIndex := 1;
        btnNext.Click;
    end else begin
        pcPages.ActivePageIndex := 0;
    end;

    result := false;
    self.SetButtons;
    self.ShowModal;
    result := self.ModalResult = mrOK;
end;

procedure TFrmGestureWizard.FormActivate(Sender: TObject);
begin
    case pcPages.ActivePageIndex of
    2: begin
        FrameKeystrokeEdit1.txtHotkey.SetFocus;
    end;    
    end;
end;

procedure TFrmGestureWizard.FormShow(Sender: TObject);
var i, ap : integer;
begin
    ap := pcPages.ActivePageIndex;
    for i := 0 to pcPages.PageCount  - 1 do
        pcPages.Pages[i].TabVisible := false;


    if not EditMode then begin
        pcPages.ActivePageIndex := 0;
    end else begin
        pcPages.ActivePageIndex := ap;
    end;
    self.SetButtons;

    if not EditMode then begin
        self.InitPages;
    end;
end;

procedure TFrmGestureWizard.InitPage;
var prog, param : string;
    kd : TKeystrokeData;
    sl : TStringList;
    i : integer;
begin


    case pcPages.ActivePageIndex of
    0: begin
    end;
    2: begin
        if FrameKeystrokeEdit1.txtHotkey.canfocus and FrameKeystrokeEdit1.txtHotkey.visible and self.Visible then
            FrameKeystrokeEdit1.txtHotkey.SetFocus;
    end;
    3: begin
    end;
    4: begin
    end;
    5: begin
        mlines.Clear;
        FrameGestureEdit1.GetGesture(self.fGestureName, self.fGesture);
        mlines.Lines.Add('Gesture:   ');
        mlines.lines.add(#9 + self.fGestureName);
        mlines.lines.add(' ');
        if rbSingle.checked then begin
            self.fActionType := atKeystroke;
            FrameKeystrokeEdit1.GetHotKey(kd);
            mlines.lines.Add('Keystroke: ');
            mlines.lines.add(#9 + kd.name);

            self.fActionData := FrameKeystrokeEdit1.ToString(kd);

        end else if rbMultiple.Checked then begin
            self.fActionType := atMultiKeystroke;
            sl := TStringList.Create;
            FrameKeystrokeList1.GetKeystrokeList(sl,'');
            self.fActionData := '';
            for i := 0 to sl.count - 1 do begin
                kd := FrameKeystrokeEdit1.FromString(sl.strings[i]);
                if i = 0 then begin
                    mlines.lines.Add('Keystrokes: ');
                end;
                mlines.lines.add(#9 + kd.name);

                if self.fActionData <> '' then fActionData := fActionData + '|';
                self.fActionData := fActionData + sl.Strings[i];
            end;
            mlines.lines.add(' ');
        end else if rbRun.checked then begin
            self.fActionType := atProgram;

            FrameProgramEdit1.GetProgram(prog, param);
            mlines.Lines.Add('Program: ');
            mlines.Lines.Add(#9+ prog);
            mlines.lines.add(' ');

            if param <> '' then begin
                mlines.Lines.Add('Parameters: ');
                mlines.Lines.Add(#9+ param);
            end;
            fActionData := prog + '|' + param;

        end;

    end;


    end;

    self.afterfirst := true;
end;

procedure TFrmGestureWizard.InitPages;
begin
    FrameGestureEdit1.Init;
    FrameKeystrokeEdit1.Init;
    FrameKeystrokeList1.init;
end;

function TFrmGestureWizard.NewGesture: boolean;
begin
    self.EditMode := false;
    result := false;
    self.ShowModal;
    result := self.ModalResult = mrOK;
end;

procedure TFrmGestureWizard.SetActionData(const Value: string);
begin
    fActionData := Value;
    case fActionType  of
    atKeystroke: FrameKeystrokeEdit1.SetKeystroke(value);
    atMultiKeystroke: FrameKeystrokeList1.SetKeystrokeList(value);
    atProgram: FrameProgramEdit1.SetProgram(value);
    end;
end;

procedure TFrmGestureWizard.SetActionType(const Value: TActionType);
begin
    FActionType := Value;
    case value of
        atKeystroke: rbSingle.checked := true;
        atMultiKeystroke: rbMultiple.Checked := true;
        atProgram: rbRun.Checked := true;
    end;
end;

procedure TFrmGestureWizard.SetButtons;
begin
    btnBack.Enabled := false;
    btnNext.Enabled := false;
    btnFinish.Enabled := false;

    if pcPages.ActivePageIndex = 0 then begin
        btnNext.Enabled := true;
    end else if pcPages.ActivePageIndex = pcPages.PageCount-1 then begin
        btnFinish.Enabled := true;
        btnBack.Enabled := true;
    end else begin
        btnBack.Enabled := true;
        btnNext.Enabled := true;
    end;

end;

procedure TFrmGestureWizard.SetGesture(const Value: string);
begin
    fGesture := Value;
    FrameGestureEdit1.SetGesture(value);
end;

end.
