unit UnitFrmSysTrayMenu;
{
    Purpose:
        This unit is responsible for the system tray icon and it's
        popup menu.
    Updates:
        Updated for option to show popup on double-click

        -------------
        Show the version in the Icon "Hint"
        -------------
        Updated method of modifying the systray icon so it does change
        the systray or move in position.

        Added public method to disable/enable systray icon changing

        Figured out a simple/reliable way to show picture on menu item
        Using UnitClipQueue
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, ShellAPI, StdCtrls, ImgList, XPMan, StdActnMenus,
  ActnPopup, XPStyleActnCtrls, UnitFrmDummyUnicodeTooltip, ActnMan,
  Vcl.PlatformDefaultStyleActnCtrls;


const MY_WM_TRAYICON = WM_USER + 1; // used to receive tray icon messsages


{type TMYPopupActionBar = class(TPopupActionBar)
    public
        constructor Create(AOwner: TComponent); override;
end;
}
type
  TfrmSysTrayMenu = class(TForm)
    Label1: TLabel;
    ToggleTimer: TTimer;
    ImageList1: TImageList;
    timDisabled: TTimer;
    PopupActionBar1: TPopupActionBar;
    ilPopupIcons: TImageList;
    test1: TMenuItem;
    test21: TMenuItem;
    timCloseTooltip: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure timDisabledTimer(Sender: TObject);
    procedure timCloseTooltipTimer(Sender: TObject);
  private
    { Private declarations }

    TrayIcon: TNotifyIconData;          {tray icon info}
    iconPic : TIcon;
    DisableIconChanges : boolean;
    PopupShowing : boolean;
    WM_TASKBAR_CREATE : Cardinal;
    ForceIconRestore : boolean;
    JustSwitched : boolean;
    targetEXE : string;
    CurrentIconIndex : integer;
    DisalbedEXEs : TStringList;
    NoRightClickEXEs : TStringList;
    tooltip : TTooltipWindow;
    notooltip : boolean;

    procedure ExitMenuItemClickEvent(Sender: TObject);
    //procedure FlushHistoryMenuItemClickEvent(sender : TObject);
    procedure EditGesturesMenuItemClickEvent(sender : TObject);
     procedure EditClipboardMenuItemClickEvent(sender : TObject);
     
    procedure ConfigureMenuItemClickEvent(Sender : TObject);
    procedure RefreshMenuItemClickEvent(Sender : TObject);
    procedure FlushTextNonTextClickEvent(Sender : TObject);
    procedure FlushEverything(Sender : TObject);
    procedure FlushClipboard(Sender: TObject);
    procedure ClipboardBarShow(Sender : TObject);
    procedure ClipboardBarHide(Sender : Tobject);

    function GetIconTooltipText : string;


    procedure AboutClickEvent(sender : TObject);
    procedure SetIconByIndex(index : longint);
    procedure CreateTrayIcon;
  public
    TrayPopup : TPopupMenu;
    procedure DisabledEXEClickEvent(Sender : TObject);
        procedure NoRightClickEvent(Sender : TObject);
      procedure RemovedItemsMenuItemClickEvent(sender : Tobject);

    procedure SwitchMenuItemClickEvent(sender : TOBject);
    procedure MethodMenuItemClickEvent(Sender : TObject);
    procedure SwitchPermFolder(sender : TObject);
    function IsDisabledEXE(exename : string) : boolean;
    function IsNoRightClickEXE(exename : string) : boolean;
    procedure ToggletDisabledEXE(exename : string);
    procedure ToggleNoRightClickEXE(exename : string);
    { Public declarations }

        procedure WMTrayIcon(var Msg: TMessage); message MY_WM_TRAYICON;
        procedure WndProc(var Msg: TMessage); override; // trap new icon message

        procedure SetIconNormal;
        procedure SetIconIgnore;
        procedure SetIconTriggered;
        procedure SetDisableIconChanges(value : boolean);
        function GetJustSwitched : boolean;


        procedure ShowPopup(inmenu : TMenuItem = nil);
  end;

var
  frmSysTrayMenu: TfrmSysTrayMenu;

implementation

uses  clipbrd,

  UnitKeyboardQuery,  UnitMisc,
  UnitPaste,  ActnMenus, UnitFrmGestures, UnitFrmAbout, UnitFrmConfig,
  UnitFrmMouseMonitor;
{$R *.dfm}

CONST INDEX_IGNORE = 0;
        INDEX_NORMAL = 2;
        INDEX_TRIGGERED = 1;


{////////////////////}
{//Public Interface//}
{////////////////////}

procedure TfrmSysTrayMenu.ToggleNoRightClickEXE(exename: string);
begin
    self.targetEXE := exename;
    self.NoRightClickEvent(self);
end;

procedure TfrmSysTrayMenu.ToggletDisabledEXE(exename: string);
begin
    self.targetEXE := exename;
    self.DisabledEXEClickEvent(self);
end;

procedure TFrmSysTrayMenu.SetDisableIconChanges(value: boolean);
begin
   self.DisableIconChanges := value;
end;

procedure TFrmSysTrayMenu.SetIconNormal;
var s : string;
begin
    ToggleTimer.Enabled := false;

    {Do I really need disable updates if the icon is normal?}
    //if (not self.DisableIconChanges) or (self.ForceIconRestore) then begin
        self.SetIconByIndex(1);
       { self.CurrentIconIndex := INDEX_NORMAL;
        self.ForceIconRestore := false;
        TrayIcon.cbSize := SizeOf(TrayIcon);
        TrayIcon.Wnd := Self.Handle;
        TrayIcon.uID := 0;
        TrayIcon.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
        TrayIcon.uCallbackMessage := MY_WM_TRAYICON;
        TrayIcon.hIcon := Application.Icon.Handle;
        //TrayIcon.szTip := 'ArsClip';
        s := GetIconTooltipText;
        move(s[1], TrayIcon.szTip, length(s));


        ShellAPI.Shell_notifyIcon(NIM_MODIFY, @TrayIcon);}
    //end;
end;

procedure TFrmSysTrayMenu.SetIconIgnore;
begin
    self.SetIconByIndex(INDEX_IGNORE);
end;

procedure TFrmSysTrayMenu.SetIconTriggered;
begin
    self.SetIconByIndex(INDEX_TRIGGERED);
end;

procedure TFrmSysTrayMenu.SetIconByIndex(index : longint);
var s : string;
    pc : PChar;
begin
    ToggleTimer.Enabled := true;

    if (not self.DisableIconChanges) then begin
        self.CurrentIconIndex := index;
        ImageList1.GetIcon(index, IconPic);
        TrayIcon.cbSize := SizeOf(TrayIcon);
        TrayIcon.Wnd := Self.Handle;
        TrayIcon.uID := 0;
        TrayIcon.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
        TrayIcon.uCallbackMessage := MY_WM_TRAYICON;

        TrayIcon.hIcon :=  IconPic.Handle;
        s := GetIconTooltipText;
        {$R-}
        if s <> '' then begin
            s := s + #0;
            move(s[1], TrayIcon.szTip, length(s)*sizeof(Char));
        end;
        {$R+}


        ShellAPI.Shell_notifyIcon(NIM_MODIFY, @TrayIcon);
    end;
end;


{
--========================
-- // Create/Destroy   //
--========================
}
const DISABLED_FILE = 'disabled.ini';
const NORIGHTCLICK_FILE = 'noright.ini';
procedure TfrmSysTrayMenu.FormCreate(Sender: TObject);
var s : string;
begin
    UnitMisc.AppendLog('SystemTrayIcon - creating');

    if (true) then begin
        TrayPopup := TPopupMenu.Create(self);
    end else begin
        TrayPopup := TPopupActionBar.Create(self);
    end;
    iconpic := TIcon.Create;


    self.CreateTrayIcon;
    WM_TASKBAR_CREATE := RegisterWindowMessage('TaskbarCreated');

    self.DisalbedEXEs := TStringList.Create;
    self.NoRightClickEXEs := TStringList.Create;
    s := UnitMisc.GetAppPath;
    s := s + DISABLED_FILE;
    if FileExists(s) then begin
        self.DisalbedEXEs.LoadFromFile(s);
    end;
    s := UnitMisc.GetAppPath;
    s := s + NORIGHTCLICK_FILE;
    if FileExists(s) then begin
        self.NoRightClickEXEs.LoadFromFile(s);
    end;


    tooltip := TTooltipWindow.Create;
end;

procedure TfrmSysTrayMenu.CreateTrayIcon;
var s : string;
begin
    //
    // create the initial icon
    // NIM_MODIFY will be performed for updates
    //
    TrayIcon.cbSize := SizeOf(TrayIcon);
    TrayIcon.Wnd := Self.Handle;
    TrayIcon.uID := 0;
    TrayIcon.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
    TrayIcon.uCallbackMessage := MY_WM_TRAYICON;
    //TrayIcon.hIcon := Application.Icon.Handle;
    ImageList1.GetIcon(1, IconPic);
    TrayIcon.hIcon :=  IconPic.Handle;
    s := GetIconTooltipText;
    {$R-}
    if s <> '' then
        move(s[1], TrayIcon.szTip, length(s));
    {$R+}

    ShellAPI.Shell_notifyIcon(NIM_ADD, @TrayIcon);

end;


procedure TfrmSysTrayMenu.FormDestroy(Sender: TObject);
var s : string;
begin
    //
    // Remove tray icon
    //
    ShellAPI.Shell_notifyIcon(NIM_DELETE, @TrayIcon);

    TrayPopup.Items.Clear;
    MyFree(TrayPopup);
    MyFree(iconPic);

    s := UnitMisc.GetAppPath;
    s := s + DISABLED_FILE;
    self.DisalbedEXEs.SaveToFile(s);
    s := UnitMisc.GetAppPath;
    s := s + NORIGHTCLICK_FILE;
    self.NoRightClickEXEs.SaveToFile(s);
    MyFree(DisalbedEXEs);
    MyFree(NoRightClickEXEs);
end;






{
--========================
-- // Trayicon messages //
--========================
                                          r
Description: On a right click on the tray icon,
    present a popup menu that allows the user to

close the program
run configuration
run permanent items
switch permanent items
flush the clipboard history

}


procedure TfrmSysTrayMenu.WndProc(var Msg: TMessage);
begin
    // force a restore if explorer crashes
    if msg.Msg = self.WM_TASKBAR_CREATE  then begin
        self.ForceIconRestore := true;
        self.CreateTrayIcon;
        EXIT;
    end else begin
        inherited;
    end;

    {
    // the tray icon never receives many messages, like below
    case msg.msg of
    WM_MOUSEHOVER:
        begin
            showmessage('hover');
        end;
    else
        inherited;
    end;
    }
end;


procedure TFrmSysTrayMenu.WMTrayIcon(var Msg: TMessage);
//var ci : TClipItem;
begin
    //
    // show Popup menu on a right click
    // show configuration form on a left double click
    // show/hide debug window on right double click
    //

    case msg.LParam of
    WM_LBUTTONDOWN : begin
    end;

    WM_MOUSEMOVE: begin
        if (tooltip.IsShowing) and (not self.notooltip) then begin
            self.timCloseTooltip.Enabled := false;
            self.timCloseTooltip.Enabled := true;
        end;
    end;
    WM_Rbuttondown : begin
        self.notooltip := true;
        tooltip.CloseTooltip;
        self.ShowPopup;
        self.notooltip := false;
    end;
    WM_LBUTTONDBLCLK : begin
        self.notooltip := true;
        tooltip.CloseTooltip;

        self.EditGesturesMenuItemClickEvent(self); 
        self.notooltip := false;

    end;
    WM_RBUTTONDBLCLK : begin
    end;
    end;


end;




procedure TFrmSysTrayMenu.ShowPopup(inmenu : TMenuItem);
var CursorPos : TPoint;
    menu: TMenuItem;
    submenu : TMenuItem;
    //TrayPopup : TPopupMenu;
    i : integer;
    s : string;
    {procedure ImageToBitmap(img : TImage; bmp : TBitmap);
    begin
        bmp.pixelformat := pf24bit;
        bmp.Width := img.Width;
        bmp.Height := img.Height;
        bmp.Canvas.draw(0,0, img.Picture.Graphic);
    end;}
    procedure AddToPopup(item : TMenuItem);
    begin
        if (inmenu = nil) then begin
            TrayPopup.Items.Add(item);
        end else begin
            inmenu.Add(item);
        end;
    end;
begin
    if (self.PopupShowing) then EXIT;
    self.PopupShowing := true;


    {
    |=================|
    | Configure       |
    | Assign method to [program] |>|
    |-----------------|
    | Permenant Items |
    | Switch to... |> |
    |-----------------|
    | Flush History   |
    | Edit History    |
    | Edit Clipboard..|
    | Removed Items...|
    |-----------------|
    | About           |
    | Exit            |
    |=================|
    }
    UnitMisc.AppendLog('WMTrayIcon right button ');

    if (inmenu = nil) then begin
        Windows.GetCursorPos(CursorPos);
        //MyFree(TrayPopup);

        //TrayPopup := TPopupActionBar.Create(nil);
        //TrayPopup := TPopupMenu.Create(nil);
        TrayPopup.Items.Clear;
        TrayPopup.AutoHotkeys := maAutomatic;
    end;

    MyFree(TrayPopup);
    if (true) then begin
        TrayPopup := TPopupMenu.Create(self);
    end else begin
        TrayPopup := TPopupActionBar.Create(self);
    end;

    traypopup.Images := ilPopupIcons;
    // for dynamically created popup, the image list isn't set yet
    // and must be forced

    menu := TMenuItem.Create(TrayPopup);
    menu.caption := '&Configure ...';
    menu.OnClick := self.ConfigureMenuItemClickEvent;
    //menu.Bitmap := FrmMainPopup.imgA.Picture.Bitmap;
    menu.ImageIndex := 0;
    menu.Bitmap := nil;
    //menu.hint := IntToStr(Integer(@FrmMainPopup.imgA));
    AddToPopup(menu);




    AddToPopup(NewLine());


    menu := TMenuItem.Create(TrayPopup);
    menu.Caption := 'Edit Gestures ...';
    menu.OnClick := self.EditGesturesMenuItemClickEvent ;
    //menu.Bitmap := FrmMainPopup.imgRightArrow.Picture.Bitmap;
    menu.ImageIndex := 1;
    AddToPopup(menu);





    AddToPopup(NewLine());
    menu := TMenuItem.Create(TrayPopup);
    menu.caption := 'About';
    menu.OnClick := self.AboutClickEvent;
    AddToPopup(menu);


    menu := TMenuItem.Create(TrayPopup);
    menu.caption := 'Exit MouseWrangler';
    menu.OnClick := self.ExitMenuItemClickEvent;
    AddToPopup(menu);





    //UnitMisc.MySetTheme( GetMenu( );
    if (inmenu = nil) then begin
        UnitMisc.AppendLog('>>tray popup');
        Windows.SetForegroundWindow(FrmGestures.handle);

        { Windows.TrackPopupMenu(TrayPopup.Handle,
        TPM_LEFTBUTTON or TPM_RIGHTBUTTON or TPM_LEFTALIGN,
        CursorPos.x, CursorPos.y, 0,
        self.Handle, nil);}
        TrayPopup.Popup(CursorPos.x+2, CursorPos.y-4);
        //TrayPopup.Popup(CursorPos.x+2, CursorPos.y-4);
        // workaround for well known bug
        Windows.PostMessage(FrmGestures.Handle, WM_NULL, 0, 0);
        UnitMisc.AppendLog('>>tray popup done');

    end;


    self.PopupShowing := false;
end;


{
  // menu item clicks
}


procedure TFrmSysTrayMenu.RefreshMenuItemClickEvent(Sender : TObject);
begin

end;
procedure TFrmSysTrayMenu.ExitMenuItemClickEvent(Sender: TObject);
begin
    // End the program nicely
    FrmGestures.ExitProgram;
end;
procedure TFrmSysTrayMenu.SwitchMenuItemClickEvent(sender : TOBject);
begin
end;
procedure TfrmSysTrayMenu.ClipboardBarHide(Sender: Tobject);
begin
end;

procedure TfrmSysTrayMenu.ClipboardBarShow(Sender: TObject);
begin
end;

procedure TFrmSysTrayMenu.SwitchPermFolder(sender : TObject);
begin
end;

procedure TfrmSysTrayMenu.timCloseTooltipTimer(Sender: TObject);
begin
    tooltip.CloseTooltip;
    timCloseTooltip.Enabled := false;
end;

procedure TfrmSysTrayMenu.timDisabledTimer(Sender: TObject);
begin

    If (true) then begin
        if (KeyboardQuery.LockOn(VK_SCROLL)) then begin
            self.SetIconIgnore;
            FrmMouseMonitor.SetMonitorEnabled(false);
            EXIT;
        end;
        FrmMouseMonitor.SetMonitorEnabled(true);

    end;
    if self.CurrentIconIndex <> INDEX_NORMAL then begin
        self.SetIconNormal;
    end;
    
end;

procedure TfrmSysTrayMenu.DisabledEXEClickEvent(Sender: TObject);
var s : string;
begin
    s := self.DisalbedEXEs.values[targetEXE];
    if s = '' then begin
        self.DisalbedEXEs.values[targetEXE] := '1';
    end else begin
        self.DisalbedEXEs.values[targetEXE] := '';
    end;

    targetEXE := '';
end;


procedure TfrmSysTrayMenu.NoRightClickEvent(Sender: TObject);
var s : string;
begin
    s := self.NoRightClickEXEs.values[targetEXE];
    if s = '' then begin
        self.NoRightClickEXEs.values[targetEXE] := '1';
    end else begin
        self.NoRightClickEXEs.values[targetEXE] := '';
    end;

    targetEXE := '';

end;


{
procedure TFrmSysTrayMenu.FlushHistoryMenuItemClickEvent(sender : TObject);
begin
    if (Dialogs.MessageDlg('Flush all text and non-text items?',
        mtConfirmation, [mbYes,mbNo],0) = mrYes) then begin
        ClipQueue.ClearQueue;
        OtherQueue.ClearQueue;
    end;
end;
}
procedure TfrmSysTrayMenu.EditClipboardMenuItemClickEvent(sender: TObject);
var p : TPosition;
begin
end;

procedure TFrmSysTrayMenu.EditGesturesMenuItemClickEvent(sender : TObject);
begin
    if ThreadAttach(GetForegroundWindow)  then begin
        FrmGestures.show;
        Windows.SetForegroundWindow(Application.Handle);

        ThreadDetach;
    end;
end;

procedure TFrmSysTrayMenu.AboutClickEvent(sender : TObject);
begin
     if ThreadAttach(GetForegroundWindow)  then begin
        FrmAbout.show;
        Windows.SetForegroundWindow(Application.Handle);

        ThreadDetach;
    end;
end;

procedure TFrmSysTrayMenu.ConfigureMenuItemClickEvent(Sender : TObject);
begin

     if ThreadAttach(GetForegroundWindow)  then begin
        FrmConfig.show;
        Windows.SetForegroundWindow(Application.Handle);

        ThreadDetach;
    end;
end;



procedure TFrmSysTrayMenu.RemovedItemsMenuItemClickEvent(sender : Tobject);
begin
end;


function TfrmSysTrayMenu.GetIconTooltipText: string;
begin
    if assigned(FrmAbout) then
        result := 'MouseWrangler v' + FrmAbout.lblversion.caption + #0;
end;

function TfrmSysTrayMenu.GetJustSwitched: boolean;
begin
    result := self.JustSwitched;
    self.JustSwitched := false;
end;

procedure TfrmSysTrayMenu.MethodMenuItemClickEvent(Sender: TObject);
begin
end;





function TfrmSysTrayMenu.IsDisabledEXE(exename: string): boolean;
begin
    result := self.DisalbedEXEs.Values[exename] <> '';
end;

function TfrmSysTrayMenu.IsNoRightClickEXE(exename: string): boolean;
begin
    result := self.NoRightClickEXEs.Values[exename] <> '';
end;

procedure TfrmSysTrayMenu.FlushTextNonTextClickEvent(Sender: TObject);
begin
end;

procedure TfrmSysTrayMenu.FlushClipboard(Sender: TObject);
begin
    clipbrd.Clipboard.Clear;
end;

procedure TfrmSysTrayMenu.FlushEverything(Sender: TObject);
begin

end;


{ TMYPopupActionBar }
{
constructor TMYPopupActionBar.Create(AOwner: TComponent);
var mystyle : TXPStyleActionBars;
    colormap : TCustomActionBarColorMap;
begin
    inherited;
    self.PopupMenu := TCustomActionPopupMenuEx.Create(self);
    mystyle := self.PopupMenu.ActionManager.Style;
end;
}
end.
